@@ -462,23 +462,12 @@ def _error(what, rc):
     `rc`, using :py:class:`Error` if no better class exists."""
     return _error_map.get(rc, Error)(what, rc)
 
-def _kill_dependents(parent):
-    """Notify all dependents of `parent` that `parent` is about to become
-    invalid."""
-    deps = parent._deps
-    while deps:
-        chid, chref = deps.popitem()
-        child = chref()
-        if child:
-            child._invalidate()
-
 class Some_LMDB_Resource_That_Was_Deleted_Or_Closed(object):
     """We need this because CFFI on PyPy treats None as cffi.NULL, instead of
     throwing an exception it feeds LMDB null pointers. That means simply
-    replacing native handles with None during _kill_dependents() will lead to
-    NULL pointer derefences. Instead we use this class, and its weird name to
-    cause a TypeError, with a very obvious string appearing in the exception
-    text.
+    replacing native handles with None during _invalidate() will cause NULL
+    pointer dereferences. Instead use this class, and its weird name to cause a
+    TypeError, with a very obvious string in the exception text.
 
     The only alternatives to this are inserting a check around every single use
     of a native handle to ensure the handle is still valid prior to calling
@@ -492,16 +481,6 @@ class Some_LMDB_Resource_That_Was_Deleted_Or_Closed(object):
         return "<This used to be a LMDB resource but it was deleted or closed>"
 _invalid = Some_LMDB_Resource_That_Was_Deleted_Or_Closed()
 
-def _depend(parent, child):
-    """Mark `child` as dependent on `parent`, so its `_invalidate()` method
-    will be called before the resource associated with `parent` is
-    destroyed."""
-    parent._deps[id(child)] = weakref.ref(child)
-
-def _undepend(parent, child):
-    """Clean up `parent`'s dependency dict by removing `child` from it."""
-    parent._deps.pop(id(child), None)
-
 def _mvbuf(mv):
     """Convert a MDB_val cdata to a CFFI buffer object."""
     return _ffi.buffer(mv.mv_data, mv.mv_size)
@@ -665,7 +644,7 @@ class Environment(object):
         if rc:
             raise _error("mdb_env_create", rc)
         self._env = envpp[0]
-        self._deps = {}
+        self._deps = set()
 
         rc = _lib.mdb_env_set_mapsize(self._env, map_size)
         if rc:
@@ -714,6 +693,15 @@ class Environment(object):
             self._db = _Database(self, txn, None, False, False, True)
         self._dbs = {None: weakref.ref(self._db)}
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, _1, _2, _3):
+        self.close()
+
+    def __del__(self):
+        self.close()
+
     def close(self):
         """Close the environment, invalidating any open iterators, cursors, and
         transactions. Repeat calls to :py:meth:`close` have no effect.
@@ -722,15 +710,13 @@ class Environment(object):
         <http://symas.com/mdb/doc/group__mdb.html#ga4366c43ada8874588b6a62fbda2d1e95>`_
         """
         if self._env:
-            _kill_dependents(self)
+            while self._deps:
+                self._deps.pop()._invalidate()
             while self._spare_txns:
                 _lib.mdb_txn_abort(self._spare_txns.pop())
             _lib.mdb_env_close(self._env)
             self._env = _invalid
 
-    def __del__(self):
-        self.close()
-
     def path(self):
         """Directory path or file name prefix where this environment is
         stored.
@@ -1012,9 +998,9 @@ class Environment(object):
 class _Database(object):
     """Internal database handle."""
     def __init__(self, env, txn, name, reverse_key, dupsort, create):
-        _depend(env, self)
+        env._deps.add(self)
         self.env = env
-        self._deps = {}
+        self._deps = set()
 
         flags = 0
         if reverse_key:
@@ -1050,9 +1036,6 @@ class _Database(object):
     def _invalidate(self):
         pass
 
-    def __del__(self):
-        _undepend(self.env, self)
-
 open = Environment
 
 
@@ -1117,19 +1100,19 @@ class Transaction(object):
     _mutations = 0
 
     def __init__(self, env, db=None, parent=None, write=False, buffers=False):
-        _depend(env, self)
+        env._deps.add(self)
         self.env = env  # hold ref
         self._db = db or env._db
         self._env = env._env
         self._key = _ffi.new('MDB_val *')
         self._val = _ffi.new('MDB_val *')
         self._to_py = _mvbuf if buffers else _mvstr
-        self._deps = {}
+        self._deps = set()
 
         if parent:
             self._parent = parent
             parent_txn = parent._txn
-            _depend(parent, self)
+            parent._deps.add(self)
         else:
             parent_txn = _ffi.NULL
 
@@ -1162,12 +1145,12 @@ class Transaction(object):
 
     def _invalidate(self):
         self.abort()
+        self.env._deps.discard(self)
         self._parent = None
         self._env = _invalid
 
     def __del__(self):
         self.abort()
-        _undepend(self.env, self)
 
     def __enter__(self):
         return self
@@ -1198,7 +1181,8 @@ class Transaction(object):
         Equivalent to `mdb_drop()
         <http://symas.com/mdb/doc/group__mdb.html#gab966fab3840fc54a6571dfb32b00f2db>`_
         """
-        _kill_dependents(db)
+        while db._deps:
+            db._deps.pop()._invalidate()
         rc = _lib.mdb_drop(self._txn, db._dbi, delete)
         self._mutations += 1
         if rc:
@@ -1223,7 +1207,8 @@ class Transaction(object):
         Equivalent to `mdb_txn_commit()
         <http://symas.com/mdb/doc/group__mdb.html#ga846fbd6f46105617ac9f4d76476f6597>`_
         """
-        _kill_dependents(self)
+        while self._deps:
+            self._deps.pop()._invalidate()
         if self._write or not self._cache_spare():
             rc = _lib.mdb_txn_commit(self._txn)
             self._txn = _invalid
@@ -1240,7 +1225,8 @@ class Transaction(object):
         <http://symas.com/mdb/doc/group__mdb.html#ga73a5938ae4c3239ee11efa07eb22b882>`_
         """
         if self._txn:
-            _kill_dependents(self)
+            while self._deps:
+                self._deps.pop()._invalidate()
             if self._write or not self._cache_spare():
                 rc = _lib.mdb_txn_abort(self._txn)
                 self._txn = _invalid
@@ -1318,7 +1304,8 @@ class Transaction(object):
                 Named database to operate on. If unspecified, defaults to the
                 database given to the :py:class:`Transaction` constructor.
         """
-        return Cursor(db or self._db, self).replace(key, value)
+        with Cursor(db or self._db, self) as curs:
+            return curs.replace(key, value)
 
     def pop(self, key, db=None):
         """Use a temporary cursor to invoke :py:meth:`Cursor.pop`.
@@ -1327,7 +1314,8 @@ class Transaction(object):
                 Named database to operate on. If unspecified, defaults to the
                 database given to the :py:class:`Transaction` constructor.
         """
-        return Cursor(db or self._db, self).pop(key)
+        with Cursor(db or self._db, self) as curs:
+            return curs.pop(key)
 
     def delete(self, key, value=EMPTY_BYTES, db=None):
         """Delete a key from the database.
@@ -1445,8 +1433,8 @@ class Cursor(object):
             ...     path.append(cursor.value())
     """
     def __init__(self, db, txn):
-        _depend(db, self)
-        _depend(txn, self)
+        db._deps.add(self)
+        txn._deps.add(self)
         self.db = db # hold ref
         self.txn = txn # hold ref
         self._dbi = db._dbi
@@ -1468,15 +1456,24 @@ class Cursor(object):
     def _invalidate(self):
         if self._cur:
             _lib.mdb_cursor_close(self._cur)
-        self._cur = _invalid
-        self._dbi = _invalid
-        self._txn = _invalid
+            self.db._deps.discard(self)
+            self.txn._deps.discard(self)
+            self._cur = _invalid
+            self._dbi = _invalid
+            self._txn = _invalid
 
     def __del__(self):
-        if self._cur:
-            _lib.mdb_cursor_close(self._cur)
-        _undepend(self.db, self)
-        _undepend(self.txn, self)
+        self._invalidate()
+
+    def close(self):
+        """Close the cursor, freeing its associated resources."""
+        self._invalidate()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, _1, _2, _3):
+        self._invalidate()
 
     def key(self):
         """Return the current key."""
@@ -1969,7 +1966,7 @@ class Cursor(object):
         skipped = 0
         for key, value in items:
             rc = _lib.pymdb_cursor_put(self._cur, key, len(key),
-                                       val, len(val), flags)
+                                       value, len(value), flags)
             self.txn._mutations += 1
             added += 1
             if rc:
